function failures = runall(doplots)
% failures = runall([showplots='off'])
%
% Runs all of the figure scripts and returns a cell array of errors for any
% scripts that failed.
%
% If showplots is true, plots will be displayed as they are created (note that
% they will steal focus, which can be annoying). Otherwise, they will remain
% hidden until the script finishes (at which point they will also steal focus).
% Finally, if showplots is the string 'off', then they will never be shown.
narginchk(0, 1);
clearplots = false();
if nargin() < 1
    doplots = 'off';
end
if isequal(doplots, 'off')
    doplots = false();
    clearplots = true();
end

% Make sure user has the required packages.
book_setup('error');

% Set some local printing options if octave.
if isOctave()
    more('off');
    page_output_immediately(true(), 'local');
    page_screen_output(false(), 'local');
end

% Switch off plotting.
DefaultFigureVisible = get(0, 'DefaultFigureVisible');
if ~doplots
    set(0, 'DefaultFigureVisible', 'off'); % Don't show figures.
end
initialfigures = get(0, 'children');
getnewplots = @() setdiff(get(0, 'children'), initialfigures);

% Now actually run the examples.
fprintf('\nRunning book scripts.\n\n');
bookfigures = {
    % Only runs unique figure scripts.
    'ch1/fig-1.3';
    'ch1/fig-1.6';
    'ch2/fig-2.1';
    'ch2/fig-2.2';
    'ch2/fig-2.4';
    'ch2/fig-2.5';
    'ch2/fig-2.6';
    'ch2/fig-2.7';
    'ch2/fig-2.12';
    'ch2/fig-2.15';
    'ch2/fig-2.16';
    'ch2/fig-2.17';
    'ch2/fig-2.18';
    'ch2/fig-2.19';
    'ch2/fig-2.20';
    'ch2/fig-2.21';
    'ch2/fig-2.22';
    'ch2/fig-2.23';
    'ch2/fig-2.24';
    'ch2/fig-2.25';
    'ch2/fig-2.26';
    'ch2/fig-2.27';
    'ch2/fig-2.28';
    'ch2/fig-2.29';
    'ch2/fig-2.30';
    'ch2/fig-2.31';
    'ch3/fig-3.12';
    'ch3/fig-3.13';
    'ch4/fig-4.8';
    'ch4/fig-4.10';
    'ch4/fig-4.11';
    'ch4/fig-4.15';
    'ch4/fig-4.18';
    'ch5/fig-5.1';
    'ch5/fig-5.2';
    'ch5/fig-5.3';
    'ch5/fig-5.6';
    'ch5/fig-5.7';
    'ch5/fig-5.9';
    'ch5/fig-5.11';
    'ch5/fig-5.12';
    'ch5/fig-5.13';
    'ch5/fig-5.14';
    'ch5/fig-5.15';
    'ch5/fig-5.16';
    'ch5/fig-5.17';
};
failures = cell(0, 1);
for i = 1:length(bookfigures)
    fig = bookfigures{i};
    fprintf('* %s ... ', fig);
    [err, figtime] = runmain(fig);
    if isempty(err)
        fprintf('success');
    else
        fprintf('FAILURE');
        failures{length(failures) + 1} = struct('figure', fig, 'err', err);
    end
    fprintf(' (%.4g s)\n', figtime);
    if clearplots
        close(getnewplots()); % Close figures to save memory.
    end
end

% Make figures show up.
if ~doplots && ~clearplots
    newplots = getnewplots();
    for i = 1:length(newplots)
        set(newplots(i), 'Visible', 'on');
    end
end

% Switch plots back on.
set(0, 'DefaultFigureVisible', DefaultFigureVisible);

% Display failures if output not requested.
if nargout() == 0
    for i = 1:length(failures)
        f = failures{i};
        fprintf('*** %s ***\n', f.figure);
        disp(f.err);
        
        % Print the stack trace, but skip the last two (they will always be
        % runall, runmain, and runscript_octave/matlab.
        for j = 1:(length(f.err.stack) - 3)
            disp(f.err.stack(j));
        end
    end
end

end%function

function [err, figtime] = runmain(fig)
    % [err, figtime] = runmain(fig)
    %
    % Changes to the directory of of the given figure and runs main.m, returning
    % any error message. Afterwards, changes back to the original directory.
    olddir = cd(fig);
    dircleanup = onCleanup(@() cd(olddir));
    oldpath = path();
    pathcleanup = onCleanup(@() path(oldpath));
    
    figtime = tic();
    err = runscript('main');
    figtime = toc(figtime);
end%function

function err = runscript(script)
    % err = runscript(script)
    %
    % Attempts to run a script without displaying anything and returns any error
    % message.
    figure(); % Avoids issues with re-using old figures.
    try
        evalc(script);
        err = [];
    catch err
        % Pass.
    end
end%function

